import abc


class MinosAbstractProvider(abc.ABC):

    @abc.abstractmethod
    def read(self):
        raise NotImplementedError

    @abc.abstractmethod
    def get(self, **kwargs):
        raise NotImplementedError



import pytest

from minos.common.configuration.config import MinosConfig, MinosConfigProviders
from minos.common.exceptions import MinosConfigException


def test_config_ini_fail():
    with pytest.raises(MinosConfigException):
        MinosConfig.read('./test_fail_config.ini', MinosConfigProviders.INI)


def test_config_ini_provider_default_section():
    provider_instance = MinosConfig.read('./tests/test_config.ini', MinosConfigProviders.INI)
    returned_value = provider_instance.get(key="host", section="kafka")
    assert returned_value == "kafka.minos.run"



from enum import Enum

from minos.common.configuration.providers.ini import MinosConfigIniProvider


class MinosConfigProviders(Enum):
    INI = 1


class MinosConfig(object):

    @staticmethod
    def read(path: str, provider: MinosConfigProviders):
        """
        prepare the config class with the specified provider
        """
        if provider == MinosConfigProviders.INI:
            config = MinosConfigIniProvider(path=path)
            config.read()
        return config



import configparser
import os
import typing as t

from minos.common.configuration.providers.abstract import MinosAbstractProvider
from minos.common.exceptions import MinosConfigException


class MinosConfigIniProvider(MinosAbstractProvider):
    __slots__ = "_path", "_parser"

    def __init__(self, path: str):
        if os.path.isfile(path):
            self._path = path
            self._parser = configparser.ConfigParser()
        else:
            raise MinosConfigException(f"file {path} does not exit")

    def read(self):
        self._parser.read(self._path)

    def get(self, key: str, section: str) -> t.Any:
        # get a key from a specific section
        if self._parser.has_section(section):
            # check if the key exist
            if self._parser.has_option(section, key):
                return self._parser[section][key]
            else:
                raise MinosConfigException(f"the key {key} is not in the config file")
        else:
            raise MinosConfigException(f"the section {section} does not exist")



class MinosException(Exception):
    """Exception class for import packages or modules"""
    __slots__ = ('_message')

    def __init__(self, error_message: str):
        self._message = error_message

    def __str__(self) -> str:
        """represent in a string format the error message passed during the instantiation"""
        return self._message


class MinosImportException(MinosException): pass
class MinosProtocolException(MinosException): pass
class MinosMessageException(MinosException): pass
class MinosConfigException(MinosException): pass



import pytest

from minos.common.protocol.avro import MinosAvroProtocol, MinosAvroValuesDatabase


def test_encoder_decoder_with_body_dict():
    headers = {'id': 123, 'action': 'get'}
    body = {'message': "this is a message"}
    data_return_bytes = MinosAvroProtocol.encode(headers, body)
    assert isinstance(data_return_bytes, bytes) == True

    # decode
    returned_dict = MinosAvroProtocol.decode(data_return_bytes)
    assert returned_dict['headers']['action'] == 'get'
    assert returned_dict['headers']['id'] == 123
    assert returned_dict['body']['message'] == "this is a message"


def test_encoder_decoder_with_body_string():
    headers = {'id': 123, 'action': 'get'}
    body = "this is a message"
    data_return_bytes = MinosAvroProtocol.encode(headers, body)
    assert isinstance(data_return_bytes, bytes) == True

    # decode
    returned_dict = MinosAvroProtocol.decode(data_return_bytes)
    assert returned_dict['headers']['action'] == 'get'
    assert returned_dict['headers']['id'] == 123
    assert returned_dict['body'] == "this is a message"


def test_encoder_decoder_with_body_int():
    headers = {'id': 123, 'action': 'get'}
    body = 222
    data_return_bytes = MinosAvroProtocol.encode(headers, body)
    assert isinstance(data_return_bytes, bytes) == True

    # decode
    returned_dict = MinosAvroProtocol.decode(data_return_bytes)
    assert returned_dict['headers']['action'] == 'get'
    assert returned_dict['headers']['id'] == 123
    assert returned_dict['body'] == 222


def test_encoder_decoder_with_body_array():
    headers = {'id': 123, 'action': 'get'}
    body = [
        "testing",
        "testing2"
    ]
    data_return_bytes = MinosAvroProtocol.encode(headers, body)
    assert isinstance(data_return_bytes, bytes) == True

    # decode
    returned_dict = MinosAvroProtocol.decode(data_return_bytes)
    assert returned_dict['headers']['action'] == 'get'
    assert returned_dict['headers']['id'] == 123
    assert returned_dict['body'][0] == "testing"
    assert returned_dict['body'][1] == "testing2"


def test_encoder_decoder_without_body():
    headers = {'id': 123, 'action': 'get'}
    data_return_bytes = MinosAvroProtocol.encode(headers)
    assert isinstance(data_return_bytes, bytes) == True

    # decode
    returned_dict = MinosAvroProtocol.decode(data_return_bytes)
    assert returned_dict['headers']['action'] == 'get'
    assert returned_dict['headers']['id'] == 123


def test_encoder_decoder_database_values_string():
    value = "String Test"
    data_return_bytes = MinosAvroValuesDatabase.encode(value)
    assert isinstance(data_return_bytes, bytes) == True

    # decode
    returned_string = MinosAvroValuesDatabase.decode(data_return_bytes)
    assert returned_string == "String Test"


def test_encoder_decoder_database_values_dictionary():
    value = {"first": "this is a string", "second": 123}
    data_return_bytes = MinosAvroValuesDatabase.encode(value)
    assert isinstance(data_return_bytes, bytes) == True

    # decode
    returned_string = MinosAvroValuesDatabase.decode(data_return_bytes)
    assert returned_string["first"] == "this is a string"
    assert returned_string["second"] == 123


def test_encoder_decoder_database_values_list():
    value = ["first", "second"]
    data_return_bytes = MinosAvroValuesDatabase.encode(value)
    assert isinstance(data_return_bytes, bytes) == True

    # decode
    returned_string = MinosAvroValuesDatabase.decode(data_return_bytes)
    assert returned_string[0] == "first"
    assert returned_string[1] == "second"
